"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_extra_1 = require("fs-extra");
var inquirer = require("inquirer");
var path_1 = require("path");
var new_questions_1 = require("./new.questions");
var utils_1 = require("./utils");
var TEMPLATE_VARS = ['\\[SCOPE\\]', '\\[SCOPE_NOAT\\]', '\\[NAME\\]'];
var TEMPLATE_FILES = [
    'package.json',
    'build/index.ts',
    'CHANGELOG.md',
    'modules/core/package.json',
    'modules/core/README.md',
    'modules/core/rollup.config.js',
    'modules/core/tsconfig-build.json',
    'modules/feature/package.json',
    'modules/feature/README.md',
    'modules/feature/rollup.config.js',
    'modules/feature/tsconfig-build.json',
    'modules/feature/src/basket.module.ts'
];
var DOT_FILES = ['_angular-cli.json', '_editorconfig', '_gitignore'];
/**
 * Replace occurencies of the KEY with the value passed
 */
function replace(filePath, KEY, value) {
    return __awaiter(this, void 0, void 0, function () {
        var index, regex, res;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, fs_extra_1.readFile(filePath, 'utf8')];
                case 1:
                    index = _a.sent();
                    regex = new RegExp("" + KEY, 'g');
                    res = index.replace(regex, value);
                    return [2 /*return*/, fs_extra_1.writeFile(filePath, res)];
            }
        });
    });
}
function renameDotFiles(basePath) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2 /*return*/, Promise.all(DOT_FILES.map(function (file) {
                    return fs_extra_1.move(path_1.join(basePath, file), path_1.join(basePath, file.replace('_', '.')));
                }))];
        });
    });
}
/**
 * Replace occurencies of all the TEMPLATE_VARS inside a file
 * with the matching values passed.
 */
function replaceForFile(filePath, values) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2 /*return*/, TEMPLATE_VARS.reduce(function (promise, key, index) {
                    return promise.then(function () { return replace(filePath, key, values[index]); });
                }, Promise.resolve())];
        });
    });
}
function newCommand(args, options, logger) {
    return __awaiter(this, void 0, void 0, function () {
        var answers, values;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, utils_1.execTask("Create directory " + args.name, function () { return fs_extra_1.ensureDir(args.name); })];
                case 1:
                    _a.sent();
                    return [4 /*yield*/, utils_1.execTask("Copy files into " + args.name, function () {
                            return fs_extra_1.copy(__dirname + "/../templates/project", args.name);
                        })];
                case 2:
                    _a.sent();
                    return [4 /*yield*/, inquirer.prompt(new_questions_1.default)];
                case 3:
                    answers = _a.sent();
                    values = [answers.scope, answers.scope.replace('@', ''), args.name];
                    return [4 /*yield*/, renameDotFiles(args.name)];
                case 4:
                    _a.sent();
                    return [4 /*yield*/, utils_1.execTask('Replace templates', function () {
                            return Promise.all(TEMPLATE_FILES.map(function (file) { return replaceForFile(path_1.join(args.name, file), values); }));
                        })];
                case 5:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    });
}
exports.newCommand = newCommand;
